@@ -1,6 +1,23 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2011 Roberto Longobardi
+# Copyright (C) 2010-2011 Roberto Bordolanghi
+# 
+# This file is part of the Test Manager plugin for Trac.
+# 
+# The Test Manager plugin for Trac is free software: you can 
+# redistribute it and/or modify it under the terms of the GNU 
+# General Public License as published by the Free Software Foundation, 
+# either version 3 of the License, or (at your option) any later 
+# version.
+# 
+# The Test Manager plugin for Trac is distributed in the hope that it 
+# will be useful, but WITHOUT ANY WARRANTY; without even the implied 
+# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+# See the GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with the Test Manager plugin for Trac. See the file LICENSE.txt. 
+# If not, see <http://www.gnu.org/licenses/>.
 #
 
 import copy
@@ -170,7 +187,7 @@ class TestCatalog(AbstractTestDescription):
         tc_search = TestCase(self.env)
         tc_search['page_name'] = self.values['page_name'] + '_TC%'
         
-        for tc in tc_search.list_matching_objects(False, db):
+        for tc in tc_search.list_matching_objects(exact_match=False, db=db):
             if plan_id is None:
                 yield tc
             else:
